// Routines to move data to and from the editors clipboard and OS X clipboard
// Copyright (C) 2009 Fred Allen.

// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".


#include "includes.h"

bool ExportClipboard()
//	This is called from the editor any time something is moved to its clipboard.
{
	EDITOR_BUFFER* theClipboard;
	CHUNK_HEADER* theChunk;
	UINT8* theBuffer;
	UINT32 bytesToExtract;
	UINT32 theOffset;
	bool result;

	if (g_currentPasteboard == nil)
	{
		g_currentPasteboard = [NSPasteboard generalPasteboard];
	}
	result = false;

	[g_currentPasteboard clearContents];
	if ((theClipboard = EditorStartExportClipboard()))
	{
		if ((theBuffer = (UINT8*)MNewPtr(theClipboard->textUniverse->totalBytes + 1)))
		{
			theChunk = theClipboard->textUniverse->firstChunkHeader;
			theOffset = 0;
			bytesToExtract = theClipboard->textUniverse->totalBytes;
			if (ExtractUniverseText(theClipboard->textUniverse, theChunk, theOffset, theBuffer, bytesToExtract, &theChunk, &theOffset))
			{
				theBuffer[bytesToExtract] = '\0';
				NSString* pasteStr = [NSString stringWithCString:(const char*)theBuffer encoding:NSASCIIStringEncoding];

				if ([g_currentPasteboard setString:pasteStr forType:NSPasteboardTypeString])
				{
					g_pasteBoardChangeCount = g_currentPasteboard.changeCount;
					result = true;
				}
			}
			MDisposePtr(theBuffer);
		}
		EditorEndExportClipboard();
	}
	return (result);
}

NSString* replaceCRLFWithLF(NSString* inStr)
{
	NSRegularExpression* regexp	= [NSRegularExpression regularExpressionWithPattern:@"(\r\n)|(\n\r)|(\r)|(\n)" options:0 error:nil];
	return [regexp stringByReplacingMatchesInString:inStr options:0 range:NSMakeRange(0, inStr.length) withTemplate:@"\n"];
}

bool ImportClipboard()
// When the editor is about to do a paste, or something else
// that requires it to look at the contents of the clipboard
// it will call this to possibly import outside contents into
// the current clipboard
// NOTE: this should return TRUE if it succeeds, FALSE if it fails
// NOTE: if no importing is done, this should still return TRUE
{
	EDITOR_BUFFER* theClipboard;
	UINT8* clipBoardData;
	int size;
	bool fail;

	fail = false;
	if (g_currentPasteboard == nil)
	{
		g_currentPasteboard = [NSPasteboard generalPasteboard];
		g_pasteBoardChangeCount = g_currentPasteboard.changeCount - 1;
	}
	if (g_pasteBoardChangeCount != g_currentPasteboard.changeCount)
	{
		g_pasteBoardChangeCount = g_currentPasteboard.changeCount;
		NSString* clipStr = [g_currentPasteboard stringForType:NSPasteboardTypeString];
		if (clipStr)
		{
			replaceCRLFWithLF(clipStr);
			NSData* asciiData = [clipStr dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];

			if (asciiData)
			{
				size = (int)[asciiData length];
				if ((theClipboard = EditorStartImportClipboard()))
				{
					if ((clipBoardData = (UINT8*)MNewPtr(size + 1)))
					{
						UINT8* data = (UINT8*)asciiData.bytes;

						for (int i = 0; i < size; i++)
						{
							clipBoardData[i] = data[i];
						}
						clipBoardData[size] = '\0';
						EditorStartReplace(theClipboard);
						if (!ReplaceEditorText(theClipboard, 0, theClipboard->textUniverse->totalBytes, clipBoardData, size, false))
						{
							fail = true;
						}
						EditorEndReplace(theClipboard);
						MDisposePtr(clipBoardData);
					}
					else
					{
						fail = true;
					}
					EditorEndImportClipboard();
				}
				else
				{
					fail = true;
				}
			}
		}
	}
	return (!fail);
}


