// Globals needed by the GUI
// Copyright (C) 2009 Fred Allen.
//
// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

#include "includes.h"
#import <AppKit/AppKit.h>

WINDOWLISTELEMENT* g_windowListHead;	// head of the window list
bool g_hotKeyPressed;					// when set true, signals that the hotkey was pressed
NSInteger g_pasteBoardChangeCount;
NSPasteboard* g_currentPasteboard;				// the current OS X pasteboard

//
//	Table used in the menu code to convert from one of our key codes back to OS X virtual code
//
INT32 g_menuVirtKeyArray[] =
{
	kVK_ANSI_KeypadEnter,						// 0
	-1,											// 1
	-1,											// 2
	-1,											// 3
	-1,											// 4
	-1,											// 5
	-1,											// 6
	-1,											// 7
	kVK_Delete,									// 8
	kVK_Tab,									// 9
	-1,											// 10
	-1,											// 11
	kVK_ANSI_KeypadClear,						// 12
	kVK_Return,									// 13
	-1,											// 14
	-1,											// 15
	kVK_Shift,									// 16
	kVK_Control,								// 17
	-1,											// 18
	-1,											// 19
	kVK_CapsLock,								// 20
	-1,											// 21
	-1,											// 22
	-1,											// 23
	-1,											// 24
	-1,											// 25
	-1,											// 26
	kVK_Escape,									// 27
	-1,											// 28
	-1,											// 29
	-1,											// 30
	-1,											// 31
	kVK_Space,									// 32
	kVK_PageUp,									// 33
	kVK_PageDown,								// 34
	kVK_End,									// 35
	kVK_Home,									// 36
	kVK_LeftArrow,								// 37
	kVK_UpArrow,								// 38
	kVK_RightArrow,								// 39
	kVK_DownArrow,								// 40
	-1,											// 41
	-1,											// 42
	-1,											// 43
	-1,											// 44
	-1,											// 45
	kVK_ForwardDelete,							// 46
	kVK_Help,									// 47
	kVK_ANSI_0,									// 48
	kVK_ANSI_1,									// 49
	kVK_ANSI_2,									// 50
	kVK_ANSI_3,									// 51
	kVK_ANSI_4,									// 52
	kVK_ANSI_5,									// 53
	kVK_ANSI_6,									// 54
	kVK_ANSI_7,									// 55
	kVK_ANSI_8,									// 56
	kVK_ANSI_9,									// 57
	-1,											// 58
	-1,											// 59
	-1,											// 60
	-1,											// 61
	-1,											// 62
	-1,											// 63
	-1,											// 64
	kVK_ANSI_A,									// 65
	kVK_ANSI_B,									// 66
	kVK_ANSI_C,									// 67
	kVK_ANSI_D,									// 68
	kVK_ANSI_E,									// 69
	kVK_ANSI_F,									// 70
	kVK_ANSI_G,									// 71
	kVK_ANSI_H,									// 72
	kVK_ANSI_I,									// 73
	kVK_ANSI_J,									// 74
	kVK_ANSI_K,									// 75
	kVK_ANSI_L,									// 76
	kVK_ANSI_M,									// 77
	kVK_ANSI_N,									// 78
	kVK_ANSI_O,									// 79
	kVK_ANSI_P,									// 80
	kVK_ANSI_Q,									// 81
	kVK_ANSI_R,									// 82
	kVK_ANSI_S,									// 83
	kVK_ANSI_T,									// 84
	kVK_ANSI_U,									// 85
	kVK_ANSI_V,									// 86
	kVK_ANSI_W,									// 87
	kVK_ANSI_X,									// 88
	kVK_ANSI_Y,									// 89
	kVK_ANSI_Z,									// 90
	-1,											// 91
	-1,											// 92
	-1,											// 93
	-1,											// 94
	-1,											// 95
	kVK_ANSI_Keypad0,							// 96
	kVK_ANSI_Keypad1,							// 97
	kVK_ANSI_Keypad2,							// 98
	kVK_ANSI_Keypad3,							// 99
	kVK_ANSI_Keypad4,							// 100
	kVK_ANSI_Keypad5,							// 101
	kVK_ANSI_Keypad6,							// 102
	kVK_ANSI_Keypad7,							// 103
	kVK_ANSI_Keypad8,							// 104
	kVK_ANSI_Keypad9,							// 105
	kVK_ANSI_KeypadMultiply,					// 106
	kVK_ANSI_KeypadPlus,						// 107
	-1,											// 108
	kVK_ANSI_KeypadMinus,						// 109
	kVK_ANSI_KeypadDecimal,						// 110
	kVK_ANSI_KeypadDivide,						// 111
	kVK_F1,										// 112
	kVK_F2,										// 113
	kVK_F3,										// 114
	kVK_F4,										// 115
	kVK_F5,										// 116
	kVK_F6,										// 117
	kVK_F7,										// 118
	kVK_F8,										// 119
	kVK_F9,										// 120
	kVK_F10,									// 121
	kVK_F11,									// 122
	kVK_F12,									// 123
	kVK_F13,									// 124
	kVK_F14,									// 125
	kVK_F15,									// 126
	kVK_F16,									// 127
	kVK_F17,									// 128
	kVK_F18,									// 129
	kVK_F19,									// 130
	kVK_F20,									// 131
	-1,											// 132
	-1,											// 133
	-1,											// 134
	-1,											// 135
	-1,											// 136
	-1,											// 137
	-1,											// 138
	-1,											// 139
	-1,											// 140
	-1,											// 141
	-1,											// 142
	-1,											// 143
	-1,											// 144
	-1,											// 145
	-1,											// 146
	-1,											// 147
	-1,											// 148
	-1,											// 149
	-1,											// 150
	-1,											// 151
	-1,											// 152
	-1,											// 153
	-1,											// 154
	-1,											// 155
	-1,											// 156
	-1,											// 157
	-1,											// 158
	-1,											// 159
	-1,											// 160
	-1,											// 161
	-1,											// 162
	-1,											// 163
	-1,											// 164
	-1,											// 165
	-1,											// 166
	-1,											// 167
	-1,											// 168
	-1,											// 169
	-1,											// 170
	-1,											// 171
	-1,											// 172
	-1,											// 173
	-1,											// 174
	-1,											// 175
	-1,											// 176
	-1,											// 177
	-1,											// 178
	-1,											// 179
	-1,											// 180
	-1,											// 181
	-1,											// 182
	-1,											// 183
	-1,											// 184
	-1,											// 185
	kVK_ANSI_Semicolon,							// 186
	-1,											// 187
	kVK_ANSI_Comma,								// 188
	-1,											// 189
	kVK_ANSI_Period,							// 190
	kVK_ANSI_Slash,								// 191
	kVK_ANSI_Grave,								// 192
	-1,											// 193
	-1,											// 194
	-1,											// 195
	-1,											// 196
	-1,											// 197
	-1,											// 198
	-1,											// 199
	-1,											// 200
	-1,											// 201
	-1,											// 202
	-1,											// 203
	-1,											// 204
	-1,											// 205
	-1,											// 206
	-1,											// 207
	-1,											// 208
	-1,											// 209
	-1,											// 210
	-1,											// 211
	-1,											// 212
	-1,											// 213
	-1,											// 214
	-1,											// 215
	-1,											// 216
	-1,											// 217
	-1,											// 218
	kVK_ANSI_LeftBracket,						// 219
	kVK_ANSI_Backslash,							// 220
	kVK_ANSI_RightBracket,						// 221
	-1,											// 222
	-1,											// 223
	-1,											// 224
	-1,											// 225
	-1,											// 226
	-1,											// 227
	-1,											// 228
	-1,											// 229
	-1,											// 230
	-1,											// 231
	-1,											// 232
	-1,											// 233
	-1,											// 234
	-1,											// 235
	-1,											// 236
	-1,											// 237
	-1,											// 238
	-1,											// 239
	-1,											// 240
	-1,											// 241
	-1,											// 242
	-1,											// 243
	-1,											// 244
	-1,											// 245
	-1,											// 246
	-1,											// 247
	-1,											// 248
	-1,											// 249
	-1,											// 250
	-1,											// 251
	-1,											// 252
	-1,											// 253
	-1,											// 254
	-1											// 255
};

const unichar g_virtKeyToKeyEquivalentArray[] =
{
	3,											// 0
	0,											// 1
	0,											// 2
	0,											// 3
	0,											// 4
	0,											// 5
	0,											// 6
	0,											// 7
	NSDeleteCharFunctionKey,					// 8
	'	',										// 9
	0,											// 10
	0,											// 11
	0,											// 12
	13,											// 13
	0,											// 14
	0,											// 15
	0,											// 16
	0,											// 17
	0,											// 18
	0,											// 19
	0,											// 20
	0,											// 21
	0,											// 22
	0,											// 23
	0,											// 24
	0,											// 25
	0,											// 26
	27,											// 27
	0,											// 28
	0,											// 29
	0,											// 30
	0,											// 31
	' ',										// 32
	NSPageUpFunctionKey,						// 33
	NSPageDownFunctionKey,						// 34
	NSEndFunctionKey,							// 35
	NSHomeFunctionKey,							// 36
	NSLeftArrowFunctionKey,						// 37
	NSUpArrowFunctionKey,						// 38
	NSRightArrowFunctionKey,					// 39
	NSDownArrowFunctionKey,						// 40
	0,											// 41
	0,											// 42
	0,											// 43
	0,											// 44
	0,											// 45
	NSDeleteFunctionKey,						// 46
	NSHelpFunctionKey,							// 47
	'0',										// 48
	'1',										// 49
	'2',										// 50
	'3',										// 51
	'4',										// 52
	'5',										// 53
	'6',										// 54
	'7',										// 55
	'8',										// 56
	'9',										// 57
	0,											// 58
	0,											// 59
	0,											// 60
	0,											// 61
	0,											// 62
	0,											// 63
	0,											// 64
	'a',										// 65
	'b',										// 66
	'c',										// 67
	'd',										// 68
	'e',										// 69
	'f',										// 70
	'g',										// 71
	'h',										// 72
	'i',										// 73
	'j',										// 74
	'k',										// 75
	'l',										// 76
	'm',										// 77
	'n',										// 78
	'o',										// 79
	'p',										// 80
	'q',										// 81
	'r',										// 82
	's',										// 83
	't',										// 84
	'u',										// 85
	'v',										// 86
	'w',										// 87
	'x',										// 88
	'y',										// 89
	'z',										// 90
	0,											// 91
	0,											// 92
	0,											// 93
	0,											// 94
	0,											// 95
	'0',										// 96
	'1',										// 97
	'2',										// 98
	'3',										// 99
	'4',										// 100
	'5',										// 101
	'6',										// 102
	'7',										// 103
	'8',										// 104
	'9',										// 105
	'*',										// 106
	'+',										// 107
	0,											// 108
	'-',										// 109
	'.',										// 110
	'/',										// 111
	NSF1FunctionKey,							// 112
	NSF2FunctionKey,							// 113
	NSF3FunctionKey,							// 114
	NSF4FunctionKey,							// 115
	NSF5FunctionKey,							// 116
	NSF6FunctionKey,							// 117
	NSF7FunctionKey,							// 118
	NSF8FunctionKey,							// 119
	NSF9FunctionKey,							// 120
	NSF10FunctionKey,							// 121
	NSF11FunctionKey,							// 122
	NSF12FunctionKey,							// 123
	NSF13FunctionKey,							// 124
	NSF14FunctionKey,							// 125
	NSF15FunctionKey,							// 126
	NSF16FunctionKey,							// 127
	NSF17FunctionKey,							// 128
	NSF18FunctionKey,							// 129
	NSF19FunctionKey,							// 130
	NSF20FunctionKey,							// 131
	0,											// 132
	0,											// 133
	0,											// 134
	0,											// 135
	0,											// 136
	0,											// 137
	0,											// 138
	0,											// 139
	0,											// 140
	0,											// 141
	0,											// 142
	0,											// 143
	0,											// 144
	0,											// 145
	0,											// 146
	0,											// 147
	0,											// 148
	0,											// 149
	0,											// 150
	0,											// 151
	0,											// 152
	0,											// 153
	0,											// 154
	0,											// 155
	0,											// 156
	0,											// 157
	0,											// 158
	0,											// 159
	0,											// 160
	0,											// 161
	0,											// 162
	0,											// 163
	0,											// 164
	0,											// 165
	0,											// 166
	0,											// 167
	0,											// 168
	0,											// 169
	0,											// 170
	0,											// 171
	0,											// 172
	0,											// 173
	0,											// 174
	0,											// 175
	0,											// 176
	0,											// 177
	0,											// 178
	0,											// 179
	0,											// 180
	0,											// 181
	0,											// 182
	0,											// 183
	0,											// 184
	0,											// 185
	';',										// 186
	0,											// 187
	',',										// 188
	0,											// 189
	'.',										// 190
	'/',										// 191
	'`',										// 192
	0,											// 193
	0,											// 194
	0,											// 195
	0,											// 196
	0,											// 197
	0,											// 198
	0,											// 199
	0,											// 200
	0,											// 201
	0,											// 202
	0,											// 203
	0,											// 204
	0,											// 205
	0,											// 206
	0,											// 207
	0,											// 208
	0,											// 209
	0,											// 210
	0,											// 211
	0,											// 212
	0,											// 213
	0,											// 214
	0,											// 215
	0,											// 216
	0,											// 217
	0,											// 218
	'[',										// 219
	'\\',										// 220
	']',										// 221
	0,											// 222
	0,											// 223
	0,											// 224
	0,											// 225
	0,											// 226
	0,											// 227
	0,											// 228
	0,											// 229
	0,											// 230
	0,											// 231
	0,											// 232
	0,											// 233
	0,											// 234
	0,											// 235
	0,											// 236
	0,											// 237
	0,											// 238
	0,											// 239
	0,											// 240
	0,											// 241
	0,											// 242
	0,											// 243
	0,											// 244
	0,											// 245
	0,											// 246
	0,											// 247
	0,											// 248
	0,											// 249
	0,											// 250
	0,											// 251
	0,											// 252
	0,											// 253
	0,											// 254
	0											// 255
};


const char* g_keyNameArray[] =
{
	"ENTER",									// 0
	"LBUTTON",									// 1
	"RBUTTON",									// 2
	"CANCEL",									// 3
	"MBUTTON",									// 4
	"UNDEFINED_05",								// 5
	"UNDEFINED_06",								// 6
	"UNDEFINED_07",								// 7
	"BACKSPACE",								// 8
	"TAB",										// 9
	"UNDEFINED_0A",								// 10
	"UNDEFINED_0B",								// 11
	"CLEAR",									// 12
	"RETURN",									// 13
	"UNDEFINED_0E",								// 14
	"UNDEFINED_0F",								// 15
	"SHIFT",									// 16
	"CONTROL",									// 17
	"MENU",										// 18
	"PAUSE",									// 19
	"CAPITAL",									// 20
	"RESERVED_15",								// 21
	"RESERVED_16",								// 22
	"RESERVED_17",								// 23
	"RESERVED_18",								// 24
	"RESERVED_19",								// 25
	"UNDEFINED_1A",								// 26
	"ESCAPE",									// 27
	"RESERVED_1C",								// 28
	"RESERVED_1D",								// 29
	"RESERVED_1E",								// 30
	"RESERVED_1F",								// 31
	"SPACE",									// 32
	"PAGEUP",									// 33
	"PAGEDOWN",									// 34
	"END",										// 35
	"HOME",										// 36
	"LEFT",										// 37
	"UP",										// 38
	"RIGHT",									// 39
	"DOWN",										// 40
	"SELECT",									// 41
	"OEM_2A",									// 42
	"EXECUTE",									// 43
	"SNAPSHOT",									// 44
	"INSERT",									// 45
	"DELETE",									// 46
	"HELP",										// 47
	"0",										// 48
	"1",										// 49
	"2",										// 50
	"3",										// 51
	"4",										// 52
	"5",										// 53
	"6",										// 54
	"7",										// 55
	"8",										// 56
	"9",										// 57
	"UNDEFINED_3A",								// 58
	"UNDEFINED_3B",								// 59
	"UNDEFINED_3C",								// 60
	"UNDEFINED_3D",								// 61
	"UNDEFINED_3E",								// 62
	"UNDEFINED_3F",								// 63
	"UNDEFINED_40",								// 64
	"A",										// 65
	"B",										// 66
	"C",										// 67
	"D",										// 68
	"E",										// 69
	"F",										// 70
	"G",										// 71
	"H",										// 72
	"I",										// 73
	"J",										// 74
	"K",										// 75
	"L",										// 76
	"M",										// 77
	"N",										// 78
	"O",										// 79
	"P",										// 80
	"Q",										// 81
	"R",										// 82
	"S",										// 83
	"T",										// 84
	"U",										// 85
	"V",										// 86
	"W",										// 87
	"X",										// 88
	"Y",										// 89
	"Z",										// 90
	"LOPTION",									// 91
	"ROPTION",									// 92
	"APPS",										// 93
	"UNDEFINED_5E",								// 94
	"UNDEFINED_5F",								// 95
	"NUMPAD0",									// 96
	"NUMPAD1",									// 97
	"NUMPAD2",									// 98
	"NUMPAD3",									// 99
	"NUMPAD4",									// 100
	"NUMPAD5",									// 101
	"NUMPAD6",									// 102
	"NUMPAD7",									// 103
	"NUMPAD8",									// 104
	"NUMPAD9",									// 105
	"MULTIPLY",									// 106
	"ADD",										// 107
	"SEPARATOR",								// 108
	"SUBTRACT",									// 109
	"DECIMAL",									// 110
	"DIVIDE",									// 111
	"F1",										// 112
	"F2",										// 113
	"F3",										// 114
	"F4",										// 115
	"F5",										// 116
	"F6",										// 117
	"F7",										// 118
	"F8",										// 119
	"F9",										// 120
	"F10",										// 121
	"F11",										// 122
	"F12",										// 123
	"F13",										// 124
	"F14",										// 125
	"F15",										// 126
	"F16",										// 127
	"F17",										// 128
	"F18",										// 129
	"F19",										// 130
	"F20",										// 131
	"F21",										// 132
	"F22",										// 133
	"F23",										// 134
	"F24",										// 135
	"UNASSIGNED_88",							// 136
	"UNASSIGNED_89",							// 137
	"UNASSIGNED_8A",							// 138
	"UNASSIGNED_8B",							// 139
	"UNASSIGNED_8C",							// 140
	"UNASSIGNED_8D",							// 141
	"UNASSIGNED_8E",							// 142
	"UNASSIGNED_8F",							// 143
	"NUMLOCK",									// 144
	"SCROLL",									// 145
	"UNASSIGNED_92",							// 146
	"UNASSIGNED_93",							// 147
	"UNASSIGNED_94",							// 148
	"UNASSIGNED_95",							// 149
	"UNASSIGNED_96",							// 150
	"UNASSIGNED_97",							// 151
	"UNASSIGNED_98",							// 152
	"UNASSIGNED_99",							// 153
	"UNASSIGNED_9A",							// 154
	"UNASSIGNED_9B",							// 155
	"UNASSIGNED_9C",							// 156
	"UNASSIGNED_9D",							// 157
	"UNASSIGNED_9E",							// 158
	"UNASSIGNED_9F",							// 159
	"UNASSIGNED_A0",							// 160
	"UNASSIGNED_A1",							// 161
	"UNASSIGNED_A2",							// 162
	"UNASSIGNED_A3",							// 163
	"UNASSIGNED_A4",							// 164
	"UNASSIGNED_A5",							// 165
	"UNASSIGNED_A6",							// 166
	"UNASSIGNED_A7",							// 167
	"UNASSIGNED_A8",							// 168
	"UNASSIGNED_A9",							// 169
	"UNASSIGNED_AA",							// 170
	"UNASSIGNED_AB",							// 171
	"UNASSIGNED_AC",							// 172
	"UNASSIGNED_AD",							// 173
	"UNASSIGNED_AE",							// 174
	"UNASSIGNED_AF",							// 175
	"UNASSIGNED_B0",							// 176
	"UNASSIGNED_B1",							// 177
	"UNASSIGNED_B2",							// 178
	"UNASSIGNED_B3",							// 179
	"UNASSIGNED_B4",							// 180
	"UNASSIGNED_B5",							// 181
	"UNASSIGNED_B6",							// 182
	"UNASSIGNED_B7",							// 183
	"UNASSIGNED_B8",							// 184
	"UNASSIGNED_B9",							// 185
	";",										// 186
	"OEM_BB",									// 187
	",",										// 188
	"OEM_BD",									// 189
	".",										// 190
	"/",										// 191
	"`",										// 192
	"UNASSIGNED_C1",							// 193
	"UNASSIGNED_C2",							// 194
	"UNASSIGNED_C3",							// 195
	"UNASSIGNED_C4",							// 196
	"UNASSIGNED_C5",							// 197
	"UNASSIGNED_C6",							// 198
	"UNASSIGNED_C7",							// 199
	"UNASSIGNED_C8",							// 200
	"UNASSIGNED_C9",							// 201
	"UNASSIGNED_CA",							// 202
	"UNASSIGNED_CB",							// 203
	"UNASSIGNED_CC",							// 204
	"UNASSIGNED_CD",							// 205
	"UNASSIGNED_CE",							// 206
	"UNASSIGNED_CF",							// 207
	"UNASSIGNED_D0",							// 208
	"UNASSIGNED_D1",							// 209
	"UNASSIGNED_D2",							// 210
	"UNASSIGNED_D3",							// 211
	"UNASSIGNED_D4",							// 212
	"UNASSIGNED_D5",							// 213
	"UNASSIGNED_D6",							// 214
	"UNASSIGNED_D7",							// 215
	"UNASSIGNED_D8",							// 216
	"UNASSIGNED_D9",							// 217
	"UNASSIGNED_DA",							// 218
	"[",										// 219
	"\\",										// 220
	"]",										// 221
	"'",										// 222
	"OEM_DF",									// 223
	"OEM_E0",									// 224
	"OEM_E1",									// 225
	"OEM_E2",									// 226
	"OEM_E3",									// 227
	"OEM_E4",									// 228
	"UNASSIGNED_E5",							// 229
	"OEM_E6",									// 230
	"UNASSIGNED_E7",							// 231
	"UNASSIGNED_E8",							// 232
	"OEM_E9",									// 233
	"OEM_EA",									// 234
	"OEM_EB",									// 235
	"OEM_EC",									// 236
	"OEM_ED",									// 237
	"OEM_EE",									// 238
	"OEM_EF",									// 239
	"OEM_F0",									// 240
	"OEM_F1",									// 241
	"OEM_F2",									// 242
	"OEM_F3",									// 243
	"OEM_F4",									// 244
	"OEM_F5",									// 245
	"UNASSIGNED_F6",							// 246
	"UNASSIGNED_F7",							// 247
	"UNASSIGNED_F8",							// 248
	"UNASSIGNED_F9",							// 249
	"UNASSIGNED_FA",							// 250
	"UNASSIGNED_FB",							// 251
	"UNASSIGNED_FC",							// 252
	"UNASSIGNED_FD",							// 253
	"UNASSIGNED_FE",							// 254
	"UNASSIGNED_FF"								// 255
};
